<?php
/*
Plugin Name: WP/GeoTrack
Plugin URI: http://dadabase.de/weblog/archives/2006/05/23/wordpress-geotrack-plugin
Description: This plugin will allow you to keep track of every visitor on your public WordPress site and display his/her geo-information. <a href="options-general.php?page=wp_geotrack.php">Configuration Page</a>.
Author: Johannes Lietz
Author URI: http://dadabase.de/
Version: 2.0.2
*/

/*
WP/GeoTraq Wordpress Plugin
Copyright (C) 2006 Johannes Lietz

This program is postcardware, which means you can redistribute it and/or modify it, but
if you like it, I would be pleased if you send mit a physical snail mail (kind of oldschool
geotracking ;-)) postcard to:

Johannes Lietz
Kopernikusstr. 94
D-40225 Duesseldorf
Germany

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

$filename = substr(__FILE__, strrpos(__FILE__, '/') + 1);
if (strpos($_SERVER['PHP_SELF'], $filename) !== false) 
	die('You are not allowed to access this file directly!');

DEFINE('GEOTRACK_PATH', ABSPATH.'wp-content/plugins/wp_geotrack/');
DEFINE('GEOTRACK_URL', get_bloginfo('siteurl').'/wp-content/plugins/wp_geotrack/');
DEFINE('GEOTRACK_VERSION', '2.0');

function wp_geotrack_encode($str) {
	return strtolower(get_settings('blog_charset')) == 'utf-8' ? utf8_encode($str) : $str;
}

function wp_geotrack_loadoptions($get_defaults = false) {
	$options = get_option('wp_geotrack_options');
	
	if ($options === false) {
		$options = array();
	}
	$defaults = array();
	if (!defined('GT_NOT_INSTALLED')) {
		$defaults['gt_google_map_key'] = '';
		$gi = geoip_open(GEOTRACK_PATH.'GeoLiteCity.dat', GEOIP_STANDARD);
		$record = geoip_record_by_addr($gi, $_SERVER['REMOTE_ADDR']);
		geoip_close($gi);
		$_REQUEST['record'] = $record;
		$defaults['gt_google_map_lat'] = $record->latitude;
		$defaults['gt_google_map_long'] = $record->longitude;
	}
	else {
		$defaults['gt_google_map_lat'] = 0;
		$defaults['gt_google_map_long'] = 0;
	}
	$defaults['gt_google_map_scale'] = 4;
	$defaults['gt_list_length'] = 100;
	$defaults['gt_toplist_length'] = 10;
	$defaults['gt_cleanup_days'] = 30;
	$defaults['gt_referrers_only'] = 0;
	$defaults['gt_dateformat'] = '%d.%m.%Y %H:%i';
	$defaults['gt_exclude_user_agents'] = 'bot|spider|inktomi|crawl|slurp|library|java'
		.'|scout|jakarta|seek|find|htdig|archiver|fetch|search|scooter|libwww|wget|'
		.'control|mvaclient|cfnetwork|larbin|trivial|curl';
	$defaults['gt_debug'] = 0;
	
	foreach ($defaults as $name => $value) {
		if ($get_defaults || !isset($options[$name])) {
			$options[$name] = $value;
		}
	}
	return $options;
}

if (!file_exists(GEOTRACK_PATH.'GeoLiteCity.dat')) {
	define('GT_NOT_INSTALLED', '<b style="color: red;">GeoLiteCity.dat needs to be installed at '.GEOTRACK_PATH.'GeoLiteCity.dat!</b><br />');
}

require_once GEOTRACK_PATH.'geoipcity.inc.php';
$gt_options = wp_geotrack_loadoptions();



DEFINE('DEBUG', $gt_options['gt_debug']);

//echo '$gt_options[gt_debug]'.$gt_options['gt_debug'];



// start a PHP session if not already done
if (!headers_sent()) {
	@session_start();
}

// display debugMsg data:
function debugMsg($obj) {
	if (DEBUG) {
		echo "\n<!--\n";
		print_r($obj);
		echo "\n-->\n";
	}
}

function wp_geotrack_install() {
	global $wpdb, $table_prefix;
	if (DEBUG) {
		$wpdb->show_errors();
	}
	else {
		$wpdb->hide_errors();
	}
	$sql = "CREATE TABLE IF NOT EXISTS `".$table_prefix."geotrack` (
  `row_id` int(11) NOT NULL auto_increment,
  `ip` varchar(39) NOT NULL default '',
  `country` varchar(50) default NULL,
  `country_code` varchar(2) default NULL,
  `flagname` varchar(50) NOT NULL default '',
  `city` varchar(50) default NULL,
  `latitude` float default '0',
  `longitude` float default '0',
  `uri` text NOT NULL,
  `method` varchar(10) NOT NULL default '',
  `status` int(3) NOT NULL default '0',
  `referrer` text,
  `user_agent` varchar(255) default NULL,
  `access_time` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`row_id`)
) TYPE=MyISAM;";
	$wpdb->query($sql);
	$sql = "CREATE TABLE IF NOT EXISTS `".$table_prefix."geotrack_exclude` (
`exclude_id` int(11) NOT NULL auto_increment,
`str` varchar(255) NOT NULL default '',
PRIMARY KEY  (`exclude_id`),
UNIQUE KEY `str` (`str`)
) TYPE=MyISAM;";
	$wpdb->query($sql);
	$sql = 'ALTER TABLE `'.$table_prefix.'geotrack_exclude` DROP `exclude_id`';
	$wpdb->query($sql);
	$sql = "INSERT IGNORE INTO `".$table_prefix."geotrack_exclude` VALUES ('%.kz'),('%.tripod.com'),('%.weblog.ro/%'),
	('%/track/%'),('%12.163.72.13%'),('%1place1%'),('%1xsearch%'),('%9k.com%'),('%accounting%'),('%aciphex%'),
	('%acne%'),('%actos%'),('%acyclovir%'),('%adipex%'),('%adult%'),('%advicer%'),('%aldara%'),('%aldera%'),
	('%alfuzosin%'),('%ambien%'),('%amoxil%'),('%anal%'),('%anzwers%'),('%ares%'),('%assparade%'),('%augmentation%'),
	('%backorder%'),('%ballaire%'),('%beastiality%'),('%bestiality%'),('%bestsitereviews%'),('%betting%'),
	('%bigmouthful%'),('%blackjack%'),('%bloating%'),('%blowjob%'),('%book%'),('%boom.ru%'),('%bowel%'),('%breast%'),
	('%britney%'),('%buddy4u%'),('%bukake%'),('%buy%'),('%byethost31%'),('%c0ck%'),('%cabinetrefacingideas%'),
	('%cams%'),('%carisoprodol%'),('%casino%'),('%celebrex%'),('%celebrity%'),('%celebs%'),('%cellulite%'),
	('%chat-nett%'),('%chiki-piki%'),('%cialis%'),('%cigarette%'),('%clio%'),('%cock%'),('%craps%'),('%creampie%'),
	('%credit%'),('%crescentarian%'),('%crumsa%'),('%cum%'),('%cyclobenzaprine%'),('%d4f.de%'),('%dating%'),
	('%debt%'),('%detox%'),('%diamonds%'),('%diarrhea%'),('%diazepam%'),('%diet%'),('%dmost%'),('%drug-tests%'),
	('%duty%'),('%ebony%'),('%ejaculation%'),('%emedia%'),('%enlargement%'),('%ephedrine%'),('%erodiva%'),
	('%eroti%'),('%erotic%'),('%escort%'),('%extender%'),('%fast-search-engine%'),('%fastsize%'),('%fbhosting%'),
	('%fibromyalgia%'),('%ficken%'),('%fioricet%'),('%firsttime%'),('%fisting%'),('%fradins%'),('%freakycheats%'),
	('%free%'),('%freshset%'),('%friend-finder%'),('%frogent%'),('%fuck%'),('%future-2000%'),('%gambling%'),
	('%gaming%'),('%gift-basket%'),('%gorgeouswomenoncam%'),('%great-doctor%'),('%hair-loss%'),('%hawtalta%'),
	('%health%'),('%heartworm%'),('%hentai%'),('%hgh%'),('%highprofit%'),('%hilton%'),('%hold-em%'),('%holdem%'),
	('%hustler%'),('%hydrocodone%'),('%i-web%'),('%idate%'),('%imitrex%'),('%incest%'),('%insertion%'),('%insurance%'),
	('%inthevip%'),('%isendit%'),('%iwas2%'),('%jack%'),('%juris%'),('%kalbe%'),('%kazaa%'),('%kentris%'),
	('%kitchen%'),('%knatty%'),('%kredit%'),('%kylos%'),('%learnhowtoplay%'),('%leenow%'),('%lesbian%'),('%levitra%'),
	('%lisinopril%'),('%livenet%'),('%loan%'),('%loliduck%'),('%lortab%'),('%machtnacht%'),('%macinstruct%'),
	('%maifun%'),('%massage%'),('%mature%'),('%mcdortaklar%'),('%meds%'),('%medve%'),('%meridia%'),('%metabolism%'),
	('%mikesapartment%'),('%milf%'),('%money%'),('%mortgage%'),('%mtsby%'),('%naked%'),('%nexium%'),('%nflook%'),
	('%norco%'),('%nude%'),('%nutzu%'),('%obesity%'),('%op-clan%'),('%openlistings.net%'),('%osteoarthritis%'),
	('%ownsthis%'),('%p0rn%'),('%pain-relief%'),('%payday%'),('%penis%'),('%perso.wanadoo.es%'),('%pharmacy%'),
	('%phentemin%'),('%phentermin%'),('%phuck%'),('%pill%'),('%pills%'),('%piss%'),('%playmate%'),('%poker%'),
	('%polarhome%'),('%porn%'),('%poze%'),('%pps=wm%'),('%prescription%'),('%presctiption%'),('%price%'),
	('%prji.com%'),('%pron%'),('%propecia%'),('%proscar%'),('%psoriasis%'),('%psxtreme%'),('%qpoi%'),('%qway%'),
	('%rape%'),('%reductil%'),('%referrer.com%'),('%refinance%'),('%rheumatism%'),('%ringtone%'),('%rinngebottet%'),
	('%ronnieazza%'),('%roulette%'),('%roxtet%'),('%rulo%'),('%rutranslator%'),('%sex%'),('%shemale%'),('%shop%'),
	('%sibutramine%'),('%singles%'),('%sleep-apnea%'),('%slot%'),('%slut%'),('%software%'),('%soma%'),
	('%static.net%'),('%stop-smoking%'),('%store%'),('%strip%'),('%studyinslovakia%'),('%swinger%'),('%taboo%'),
	('%taylorbow%'),('%tecrep%'),('%teen%'),('%tenuate%'),('%terashells%'),('%testosterone%'),('%texas%'),
	('%theladysroom%'),('%thyroid%'),('%tits%'),('%tramadol%'),('%treasure%'),('%ua-princeton%'),('%ultram%'),
	('%uncovered%'),('%uprima%'),('%urniture%'),('%valium%'),('%vegas%'),('%viagra%'),('%vicodin%'),('%viox%'),
	('%vrajitor%'),('%watches%'),('%webhostips%'),('%welim%'),('%ws-op%'),('%wsop%'),('%xanax%'),('%xenical%'),
	('%xhostar%'),('%xmlindustry%'),('%xxx%'),('%yahroo%'),('%yelucie%'),('%ymore%'),('%zindagi%'),
	('http://www.google.com');";
	$wpdb->query($sql);
	$sql = 'DELETE FROM '.$table_prefix.'geotrack_exclude` WHERE str = \'\'';
	$wpdb->query($sql);
	$sql = "CREATE TABLE IF NOT EXISTS `".$table_prefix."geotrack_ipblock` (
`row_id` INT( 11 ) NOT NULL auto_increment,
`ip` VARCHAR( 39 ) NOT NULL ,
PRIMARY KEY ( `row_id` ) ,
UNIQUE (
`ip`
))";
	$wpdb->query($sql);
	$sql = 'ALTER TABLE `'.$table_prefix.'geotrack_ipblock` DROP `row_id`';
	$wpdb->query($sql);
	$sql = 'ALTER TABLE `'.$table_prefix.'geotrack_ipblock` ADD `counter` INT( 11 ) NOT NULL DEFAULT \'0\';';
	$wpdb->query($sql);
	$sql = 'CREATE TABLE IF NOT EXISTS `'.$table_prefix.'geotrack_ip_whitelist` (
		`ip` VARCHAR( 39 ) NOT NULL , UNIQUE (`ip`))';
	$wpdb->query($sql);
	$sql= 'ALTER TABLE `wp_geotrack_ipblock` ADD `referrer` TEXT NOT NULL ;';
	$wpdb->query($sql);
	$sql= 'ALTER TABLE `wp_geotrack_ipblock` ADD `blocked` TINYINT(1) NOT NULL DEFAULT \'0\';';
	$wpdb->query($sql);
}

$wp_geotrack_list = array();
$wp_geotrack_copy = "<p>Powered by <a href=\"http://www.maxmind.com/app/geolitecity\">MaxMind GeoLite City</a> and <a href=\"http://dadabase.de/weblog/archives/2006/05/23/wordpress-geotrack-plugin\">Wordpress Geotrack Plugin</a>.</p>\n";

function wp_geotrack_getlist() {
	global $wpdb, $table_prefix, $wp_geotrack_list, $gt_options;
	if (DEBUG) {
		$wpdb->show_errors();
	}
	else {
		$wpdb->hide_errors();
	}
	
	if (count($wp_geotrack_list) > 0) return;
	
	$sql = 'SELECT *, DATE_FORMAT(access_time, \''.$gt_options['gt_dateformat'].'\') AS datestr
			FROM '.$table_prefix.'geotrack 
			WHERE status < 400 AND method = \'GET\'';
	if ($gt_options['gt_referrers_only']) {
		$sql .= 'AND referrer != \'\' AND referrer NOT LIKE \'%'.$_SERVER['HTTP_HOST'].'%\' ';
	}
	$sql .= 'ORDER BY access_time DESC';
			//LIMIT 0,'.($gt_options['gt_list_length'] * 5);
	debugMsg($sql);
	
	// low-level mysql-functions for performance optimization:
	$db = mysql_connect(DB_HOST, DB_USER, DB_PASSWORD);
	mysql_select_db(DB_NAME);
	$result = mysql_query($sql);
	
	while($res = mysql_fetch_array($result, MYSQL_ASSOC)) {
		if (count($wp_geotrack_list) == $gt_options['gt_list_length']) {
			break;
		}
		if (!isset($wp_geotrack_list[$res['ip']])) {
			$obj = $res;
			if (strpos($res['referrer'], $_SERVER['HTTP_HOST']) !== false) {
				$obj['referrer'] = '';
			}
			$obj['uri'] = array();
			$wp_geotrack_list[$res['ip']] = $obj;
		}
		if ($res['referrer'] != '' && strpos($res['referrer'], $_SERVER['HTTP_HOST']) === false) {
			$wp_geotrack_list[$res['ip']]['referrer'] = $res['referrer'];
		}
		$wp_geotrack_list[$res['ip']]['uri'][] = $res['uri'];
	}
	mysql_free_result($result);
}

// display a list of tracked requests
// you can use this function inside you PHP code, e.g. your sidebar etc.
function wp_geotrack_display() {
	global $wp_geotrack_list, $wp_geotrack_copy;
	
	wp_geotrack_getlist();
	$cnt = 0;
	$str = "<ol id=\"wp-geotrack-list\" class=\"wp-geotrack\">\n";
	foreach($wp_geotrack_list as $res) {
		$class = ($cnt % 2 == 0) ? ' class="alt"' : '';
		$uri = htmlspecialchars($res['uri'][count($res['uri']) - 1]); // we've got an array of pages here, so it is easy to display clickpaths
		$str .= '<li'.$class.'><img src="'.GEOTRACK_URL.'flags/'.$res['flagname'].'.png" alt="Flag of '.$res['country'].'" title="'.$res['country'].'" /> <em>['.$res['datestr'].'], '.$res['country'].' - '.$res['city'].': <a href="'.$uri.'">[Page]</a></em>';
		
		if ($res['referrer'] != 'NULL' && $res['referrer'] != '') {
			$ref = $res['referrer'];
			if (strlen($ref) > 70) {
				$ref = substr($ref, 0, 70).'...';
			}
			$str .= '<br />Referrer: <a href="'.htmlspecialchars($res['referrer']).'">'.htmlspecialchars($ref).'</a>';
		}
		
		$str .= "</li>\n";
		$cnt++;
	}
	$str .= "</ol>\n";
	$str .= $wp_geotrack_copy;
	return $str;
}

function wp_geotrack_googlemap() {
	global $wp_geotrack_list, $wp_geotrack_copy, $gt_options;
	
	wp_geotrack_getlist();
	
	$str = '<script src="http://maps.google.com/maps?file=api&amp;v=2&amp;key='.$gt_options['gt_google_map_key'].'" type="text/javascript"></script>
<div id="google-map" style="width: 100%; height: 400px" class="wp-geotrack"></div>
<script type="text/javascript">
//<![CDATA[
var points = new Array();
function addP(la, lo, city, flagname, access_time, country) {
	points.push({la:la,lo:lo,city:city,flagname:flagname,access_time:access_time,country:country});
}';
	$points = array();
	foreach($wp_geotrack_list as $res) {
		if ($res['latitude'] != 0 && $res['longitude'] != 0 && !isset($points['p'.$res['latitude'].'_'.$res['longitude']])) {
			$points['p'.$res['latitude'].'_'.$res['longitude']] = true;
			$str .=  '
addP('.$res['latitude'].','.$res['longitude'].',\''.$res['city'].'\',\''.$res['flagname'].'\',\''.$res['datestr'].'\',\''.$res['country'].'\');';
		}
	}
	$str .=  '

function doIt(x,y,z){
	map.setCenter(new GLatLng(x,y), z);
	marker.closeInfoWindow();
}
function createMarker(p, icon) {
	var marker = new GMarker(new GLatLng(p.la,p.lo), icon);
	var htmlme = "<img src=\"'.GEOTRACK_URL.'flags/" + p.flagname + ".png\" style=\"vertical-align: -5px;\" /> " + p.city + ", " + p.country + ", " + p.access_time + "<br /><center><a href=\"javascript:void(0)\" onclick=\"doIt(" + p.la + "," + p.lo + ",10)\">Zoom In</a> | <a href=\"javascript:void(0)\" onclick=\"doIt(20,-25,'.$gt_options['gt_google_map_scale'].')\">Zoom Out</a></center>";
	//alert(htmlme);
	GEvent.addListener(marker, "mouseover", function() {
		marker.openInfoWindowHtml(htmlme);
	});
	GEvent.addListener(marker, "mouseout", function() {
		marker.closeInfoWindow();
	});
	return marker;
}
function load_googlemaps() {
	if (!GBrowserIsCompatible()) return;
	map = new GMap2(document.getElementById("google-map"));
	//map.setMapType(G_HYBRID_TYPE);
	map.addControl(new GLargeMapControl());
	map.addControl(new GOverviewMapControl());
	map.addControl(new GMapTypeControl());
	map.addControl(new GScaleControl());
	map.setCenter(new GLatLng('.$gt_options['gt_google_map_lat'].','.$gt_options['gt_google_map_long'].'), '.$gt_options['gt_google_map_scale'].');
	var bounds = map.getBounds();
	var icon = new GIcon();
	icon.image = "'.GEOTRACK_URL.'img/marker.png";
	icon.shadow = "'.GEOTRACK_URL.'img/marker_shadow.png";
	icon.iconSize = new GSize(21, 22);
	icon.shadowSize = new GSize(21, 22);
	icon.iconAnchor = new GPoint(9, 10);
	icon.infoWindowAnchor = new GPoint(5, 1);
	for (var i = 0; i < points.length; i++) {
		var p = points[i];
		var marker = createMarker(p, icon);
		map.addOverlay(marker);
	}
	delete points;
}
function addLoadEvent(func) {
	var oldonload = window.onload;
	if (typeof window.onload != "function") {
		window.onload = func;
	}
	else {
		window.onload = function() {
			oldonload();
			func();
		}
	}
}
function addUnloadEvent(func) {
	var oldonunload = window.onunload;
	if (typeof window.onunload != "function") {
		window.onunload = func;
	}
	else {
		window.onunload = function() {
			oldonunload();
			func();
		}
	}
}
addLoadEvent(load_googlemaps);
addUnloadEvent(GUnload);
//]]>
</script>';
	$str .= $wp_geotrack_copy;
	return $str;
}

function wp_geotrack_topsites() {
	global $wpdb, $table_prefix, $gt_options;
	if (DEBUG) {
		$wpdb->show_errors();
	}
	else {
		$wpdb->hide_errors();
	}
	$sql = 'SELECT COUNT(*) AS cnt, uri
			FROM '.$table_prefix.'geotrack
			WHERE status < 400 AND method = \'GET\'
			GROUP BY uri
			ORDER BY cnt DESC
			LIMIT 0, '.$gt_options['gt_toplist_length'];
	$result = $wpdb->get_results($sql, ARRAY_A);
	
	$str = "<ol id=\"wp-geotrack-topsites\" class=\"wp-geotrack\">\n";
	foreach($result as $res) {
		$str .= '<li><a href="'.htmlspecialchars($res['uri']).'">'.htmlspecialchars($res['uri']).'</a> ('.$res['cnt'].")</li>\n";
	}
	$str .= "</ol>\n";
	return $str;
}

function wp_geotrack_topreferrers() {
	global $wpdb, $table_prefix, $gt_options;
	if (DEBUG) {
		$wpdb->show_errors();
	}
	else {
		$wpdb->hide_errors();
	}
	$sql = 'SELECT COUNT(*) AS cnt, referrer 
		FROM '.$table_prefix.'geotrack 
		WHERE status < 400 
			AND method = \'GET\'
			AND referrer != \'\' 
			AND referrer NOT LIKE \''.get_bloginfo('siteurl').'%\'
		GROUP BY referrer 
		ORDER BY cnt DESC 
		LIMIT 0, '.$gt_options['gt_toplist_length'];
	$result = $wpdb->get_results($sql, ARRAY_A);
	
	$str = "<ol id=\"wp-geotrack-topreferrers\" class=\"wp-geotrack\">\n";
	if (is_array($result)) {
		foreach($result as $res) {
			$ref = $res['referrer'];
			if (strlen($ref) > 70) {
				$ref = substr($ref, 0, 70).'...';
			}
			$str .= '<li><a href="'.htmlspecialchars($res['referrer']).'">'.htmlspecialchars($ref).'</a> ('.$res['cnt'].")</li>\n";
		}
	}
	$str .= "</ol>\n";
	return $str;
}

function wp_geotrack_topcountries() {
	global $wpdb, $table_prefix, $gt_options;
	if (DEBUG) {
		$wpdb->show_errors();
	}
	else {
		$wpdb->hide_errors();
	}
	$sql = 'SELECT COUNT(*) AS cnt, country, flagname
		FROM '.$table_prefix.'geotrack 
		WHERE status < 400 AND method = \'GET\'
		GROUP BY country 
		ORDER BY cnt DESC 
		LIMIT 0, '.$gt_options['gt_toplist_length'];
	$result = $wpdb->get_results($sql, ARRAY_A);
	
	$str = "<ol id=\"wp-geotrack-topcountries\" class=\"wp-geotrack\">\n";
	foreach($result as $res) {
		$str .= '<li><img src="'.GEOTRACK_URL.'flags/'.$res['flagname'].'.png" alt="Flag of '.$res['country'].'" title="'.$res['country'].'" /> '.$res['country'].' ('.$res['cnt'].")</li>\n";
	}
	$str .= "</ol>\n";
	return $str;
}

function wp_geotrack_topcities() {
	global $wpdb, $table_prefix, $gt_options;
	if (DEBUG) {
		$wpdb->show_errors();
	}
	else {
		$wpdb->hide_errors();
	}
	$sql = 'SELECT COUNT(*) AS cnt, city, country, flagname
		FROM '.$table_prefix.'geotrack 
		WHERE status < 400 AND method = \'GET\'
		GROUP BY city, country
		ORDER BY cnt DESC 
		LIMIT 0, '.$gt_options['gt_toplist_length'];
	$result = $wpdb->get_results($sql, ARRAY_A);
	
	$str = "<ol id=\"wp-geotrack-topcountries\" class=\"wp-geotrack\">\n";
	foreach($result as $res) {
		$str .= '<li><img src="'.GEOTRACK_URL.'flags/'.$res['flagname'].'.png" alt="Flag of '.$res['country'].'" title="'.$res['country'].'" /> '.$res['city'].' ('.$res['country'].') ('.$res['cnt'].")</li>\n";
	}
	$str .= "</ol>\n";
	return $str;
}

// Replaces the special Tag <wp_geotrack-list/> in any post content with the list.
function wp_geotrack_content($content) {
	if (defined('GT_NOT_INSTALLED')) {
		echo GT_NOT_INSTALLED;
		return $content;
	}
	
	if (strpos($content, '[wp-geotrack-googlemap]') !== false)
		$content = str_replace('[wp-geotrack-googlemap]', wp_geotrack_googlemap(), $content);
	if (strpos($content, '[wp-geotrack-list]') !== false)
		$content = str_replace('[wp-geotrack-list]', wp_geotrack_display(), $content);
	if (strpos($content, '[wp-geotrack-topsites]') !== false)
		$content = str_replace('[wp-geotrack-topsites]', wp_geotrack_topsites(), $content);
	if (strpos($content, '[wp-geotrack-topreferrers]') !== false)
		$content = str_replace('[wp-geotrack-topreferrers]', wp_geotrack_topreferrers(), $content);
	if (strpos($content, '[wp-geotrack-topcountries]') !== false)
		$content = str_replace('[wp-geotrack-topcountries]', wp_geotrack_topcountries(), $content);
	if (strpos($content, '[wp-geotrack-topcities]') !== false)
		$content = str_replace('[wp-geotrack-topcities]', wp_geotrack_topcities(), $content);
	return $content;
}

function wp_geotrack_init() {
	global $wpdb, $table_prefix, $gt_options;
	if (DEBUG) {
		$wpdb->show_errors();
	}
	else {
		$wpdb->hide_errors();
	}
	
	
	if (defined('GT_NOT_INSTALLED')) return;
	if (strpos($_SERVER['REQUEST_URI'], '/wp-admin/') !== false) return;
	
	
	$referrer = (isset($_SERVER['HTTP_REFERER']) && strpos($_SERVER['HTTP_REFERER'], 'http') === 0) ? addslashes($_SERVER['HTTP_REFERER']) : '';
	//$referrer = preg_replace('/\/$/', '', $referrer);
	
	$ip = addslashes($_SERVER['REMOTE_ADDR']);
	if (preg_match('/[\d]{1,3}.[\d]{1,3}.[\d]{1,3}.[\d]{1,3}/', $_SERVER['HTTP_X_FORWARDED_FOR'])) {
		$ip = addslashes($_SERVER['HTTP_X_FORWARDED_FOR']);
	}
	
	$uri = addslashes($_SERVER['REQUEST_URI']);
	//$uri = preg_replace('/\/$/', '', $uri);
	$ua = addslashes($_SERVER['HTTP_USER_AGENT']);
	
	// exclude bots etc. and clients with empty useragents:
	if ($gt_options['gt_exclude_user_agents'] != '' && (preg_match('/('.$gt_options['gt_exclude_user_agents'].')/i', $ua) || $ua == '')) {
		return;
	}
	
	// exclude referrers with cetain keywords to prevent referrer spamming:
	$sql = 'SELECT COUNT(*) AS cnt FROM '.$table_prefix.'geotrack_exclude WHERE \''.$referrer.'\' LIKE str';
	$result = $wpdb->get_results($sql, ARRAY_A);
	
	// check if tables exist:
	if (mysql_errno() == 1146 && strpos(mysql_error(), $table_prefix.'geotrack') !== false) {
		wp_geotrack_install();
		$result = $wpdb->get_results($sql, ARRAY_A);
	}
	
	$gt_blacklisted = false;
	if ($result[0]['cnt'] > 0 && strpos($referrer, $_SERVER['HTTP_HOST']) !== 0) { // referrer is black-listed
		$gt_blacklisted = true;
	}
	
	if (!file_exists(GEOTRACK_PATH.'GeoLiteCity.dat')) {
		return;
	}
	$gi = geoip_open(GEOTRACK_PATH.'GeoLiteCity.dat', GEOIP_STANDARD);
	$record = geoip_record_by_addr($gi, $_SERVER['REMOTE_ADDR']);
	geoip_close($gi);
	
	$gt_data = array();
	$gt_data['country'] = ($record->country_name != '') ? wp_geotrack_encode($record->country_name) : 'Unknown Country';
	$gt_data['country_code'] = ($record->country_code != '') ? strtolower($record->country_code) : 'un';
	$gt_data['city'] = ($record->city != '') ? wp_geotrack_encode($record->city) : 'Unknown City';
	$gt_data['latitude'] = $record->latitude;
	$gt_data['longitude'] = $record->longitude;
	$gt_data['uri'] = $uri;
	$gt_data['referrer'] = $referrer;
	$gt_data['ua'] = $ua;
	$gt_data['ip'] = $ip;
	
	$flagname = strtolower($gt_data['country']);
	$flagname = preg_replace('/[\W]/', '_', $flagname);
	$flagname = preg_replace('/[_]+/', '_', $flagname);
	$flagname = preg_replace('/_$/', '', $flagname);
	$flagname = preg_replace('/^_/', '', $flagname);
	$gt_data['flagname'] = $flagname;
	
	$_SESSION['gt_data'] = $gt_data;
	
	$sql = 'SELECT * FROM '.$table_prefix.'geotrack_ipblock WHERE ip = \''.$ip.'\' AND blocked != -1';
	$result = $wpdb->get_results($sql, ARRAY_A);
	$die = false;
	$ref = '';
	debugMsg($result);
	if (count($result) > 0) { // ip is black-listed
		$gt_blacklisted = true;
		$sql = 'UPDATE '.$table_prefix.'geotrack_ipblock SET counter = counter + 1 WHERE ip = \''.$ip.'\'';
		$wpdb->query($sql);
		$ref = $result[0]['referrer'];
		if ($result[0]['blocked'] == 1) $die = true;
	}
	else if ($gt_blacklisted) { // add ip to black-list
		$sql = 'INSERT INTO '.$table_prefix.'geotrack_ipblock (ip, counter, referrer) VALUES (\''.$ip.'\', 1, \''.$referrer.'\')';
		$wpdb->query($sql);
	}
	
	if ($gt_blacklisted) { // remove logs from that ip
		$sql = 'DELETE FROM '.$table_prefix.'geotrack WHERE ip = \''.$ip.'\'';
		$wpdb->query($sql);
		if ($result[0]['counter'] >= 10) {
			if ($result[0]['counter'] == 10) {
				$hostname = @gethostbyaddr($ip);
				$message = "There have been visits on your blog from IP:

IP:         $ip ($hostname)
Country:    ".$gt_data['country']."
City:       ".$gt_data['city']."

which claim to be referred from:

$ref

The WP-Geotrack plugin thinks, that this might be referrer spam, which means that
someone is trying to pretend that this site has set a link to your site, so that
you and your visitors might want to see where these referrer came from, but in
fact it is a porn-, spam- or fraud-site.

If the site above is porn-, spam- or fraud-site, you should block the visitor's
IP by following this link:

".get_settings('siteurl').'/wp-admin/options-general.php?page=wp_geotrack.php&block='.urlencode($ip).'

If the site above is NOT spam or porn and contains a normal link to your site,
you should remove this IP from your blacklist:

'.get_settings('siteurl').'/wp-admin/options-general.php?page=wp_geotrack.php&remove='.urlencode($ip).'


(Advanced users: if you regularly receive non-spam warnings, you can check the table
'.$table_prefix.'geotrack_exclude in your database and remove the row which causes
these warnings.)';
				@wp_mail(get_settings('admin_email'), sprintf(__('[%s] WP-Geotrack: Suspicious IP'), get_settings('blogname')), $message);
			}
			if ($die && !headers_sent()) {
				header('HTTP/1.0 403 Bad referrer');
				die('<html><head><title>HTTP/1.0 403 Bad referrer</title></head>
					<body><h1>HTTP/1.0 403 Bad referrer</h1>
					<p>Sorry, your IP '.$ip.' has been blocked due to misuse of this service!
					<br>Please drop a quick note the server admin <a href="'.$_SERVER['SERVER_ADMIN'].'">'.$_SERVER['SERVER_ADMIN'].'</a> if you think this is a mistake!
					</body></html>');
			}
		}
	}
}

// The logging-function that is call by Wordpress inside the header of each page except admin pages.
function wp_geotrack_log() {
	global $wpdb, $table_prefix, $gt_options;
	if (DEBUG) {
		$wpdb->show_errors();
	}
	else {
		$wpdb->hide_errors();
	}
	
	if(count($_SESSION['gt_data']) == 0) { 
		return;
	}
	
	
	debugMsg($_SESSION);
	
	$status = 0;
	if (function_exists('headers_list')) {
		$headers = headers_list();
		foreach($headers as $header) {
			preg_match('/^Status: ([\d]{3})/',$header,$status_arr);
			if (isset($status_arr[1])) {
				$status = $status_arr[1];
				break;
			}
		}
	}
	
	$sql = 'INSERT INTO '.$table_prefix.'geotrack
		SET
			country = \''.$_SESSION['gt_data']['country'].'\',
			country_code = \''.$_SESSION['gt_data']['country_code'].'\',
			flagname = \''.$_SESSION['gt_data']['flagname'].'\',
			city = \''.$_SESSION['gt_data']['city'].'\',
			latitude = \''.$_SESSION['gt_data']['latitude'].'\',
			longitude = \''.$_SESSION['gt_data']['longitude'].'\',
			uri = \''.$_SESSION['gt_data']['uri'].'\',
			method = \''.$_SERVER['REQUEST_METHOD'].'\',
			status = '.$status.',
			referrer = \''.$_SESSION['gt_data']['referrer'].'\',
			user_agent = \''.$_SESSION['gt_data']['ua'].'\',
			access_time = NOW(),
			ip = \''.$_SESSION['gt_data']['ip'].'\'';
	$wpdb->query($sql);
	debugMsg($sql);
	// Purge old entries form db:
	if ($gt_options['gt_cleanup_days'] > 0) {
		$sql = 'DELETE FROM '.$table_prefix.'geotrack WHERE UNIX_TIMESTAMP(access_time) < UNIX_TIMESTAMP() - '.$gt_options['gt_cleanup_days'].' * 86400';
		$wpdb->query($sql);
	}
}

function wp_geotrack_add_menu() {
	if (function_exists('add_options_page')) {
		add_options_page(
			'WP-Geotrack Plugin Options', 
			'WP-Geotrack Plugin', 
			9, 
			basename(__FILE__), 
			'wp_geotrack_admin_menu'
		);
	}
}

function wp_geotrack_admin_menu() {
	global $wpdb, $table_prefix;
	
	$options = wp_geotrack_loadoptions(isset($_POST['restore_defaults']));
	
	if (isset($_GET['block'])) {
		$sql = 'UPDATE '.$table_prefix.'geotrack_ipblock SET blocked = 1 WHERE ip = \''.$_GET['block'].'\'';
		$wpdb->query($sql);
		echo '<div id="message" class="updated fade"><p><strong>IP '.$_GET['block'].' has been blocked!</strong></p></div>';
	}	
	else if (isset($_GET['remove'])) {
		$sql = 'DELETE FROM '.$table_prefix.'geotrack_ipblock WHERE ip = \''.$_GET['remove'].'\'';
		$wpdb->query($sql);
		echo '<div id="message" class="updated fade"><p><strong>IP '.$_GET['remove'].' has been removed from your blacklist!</strong></p></div>';
	}

	if (isset($_POST['info_update'])) {
		$fields = array('gt_google_map_key', 'gt_google_map_lat', 'gt_google_map_long', 'gt_google_map_scale', 'gt_list_length', 'gt_toplist_length', 'gt_referrers_only', 'gt_cleanup_days', 'gt_dateformat', 'gt_exclude_user_agents', 'gt_debug');
		foreach ($fields as $field) {
			$options[$field] = $_POST[$field];
		}
		$options['gt_exclude_user_agents'] = preg_replace('/\s+/', '|', $options['gt_exclude_user_agents']);
	?>
		<div id="message" class="updated fade"><p><strong>Options saved successfully!</strong></p></div><?php
	}
	else if (isset($_POST['restore_defaults'])) {
		echo '<div id="message" class="updated fade"><p><strong>Defaults restored successfully!</strong></p></div>';
	}
	else if (isset($_POST['install_db'])) {
		wp_geotrack_install();
		echo '<div id="message" class="updated fade"><p><strong>Installed MySQL Database Tables</strong></p></div>';
	}
	if (!file_exists(GEOTRACK_PATH.'GeoLiteCity.dat')) {
		echo '<div class="error"><strong>The Geo-IP Database is not installed yet!</strong><br /> Please intall it by clicking "Install/Update Geo-IP Database" below.
			If this fails, you can download the file <a href="http://www.maxmind.com/download/geoip/database/GeoLiteCity.dat.gz">here</a>, 
			unpack it (e.g. with WinZip, Stuffit Expander oder GunZip) and upload it to the wp_geotrack-folder inside your plugins-folder.</div>';
	}
	
	update_option('wp_geotrack_options',$options);
	$_SESSION['GT_UPDATE_ALLOWED'] = true;
	?>
	<div class="wrap">
		<script type="text/javascript">
		
var http_request = false;
var update_running = false;

function updateGeoDB() {
	if (update_running) {
		alert("Update is already initiated! Please Wait!");
		return;
	}
	var url = "<?php echo GEOTRACK_URL; ?>update_db.php";
	document.getElementById("update_db").value = "Updating Geo-DB, please wait...";
	alert("Updating Geo-DB, please wait, this may take some time...");
	ajaxRequest(url, alertInhalt);
	update_running = true;
}

function ajaxRequest(url, onUpdateFn) {
	
	http_request = false;

	if (window.XMLHttpRequest) { // Mozilla, Safari,...
		http_request = new XMLHttpRequest();
		if (http_request.overrideMimeType) {
			http_request.overrideMimeType('text/xml');
			// zu dieser Zeile siehe weiter unten
		}
	} else if (window.ActiveXObject) { // IE
		try {
			http_request = new ActiveXObject("Msxml2.XMLHTTP");
		} catch (e) {
			try {
				http_request = new ActiveXObject("Microsoft.XMLHTTP");
			} catch (e) {}
		}
	}

	if (!http_request) {
		alert('Error :( Cannot create XMLHTTP-Instance');
		return false;
	}
	http_request.onreadystatechange = onUpdateFn;
	http_request.open('GET', url, true);
	http_request.send(null);
}

function alertInhalt() {
	if (http_request.readyState == 4) {
		if (http_request.status == 200) {
			var xmldoc = http_request.responseXML;
			var root_node = xmldoc.getElementsByTagName('msg').item(0);
			var msg = root_node.firstChild.data;
			document.getElementById("update_db").value = msg;
			alert(msg);
		} else {
			alert('An error has occured!');
		}
	}
	update_running = false;
}

function checkForUpdate() {
	ajaxRequest("http://dadabase.de/weblog/wp-content/plugins/wp_geotrack_latestversion.xml?t=" + new Date().getTime(), alertUpdate);
}

function alertUpdate() {
	if (http_request.readyState == 4) {
		if (http_request.status == 200) {
			var xmldoc = http_request.responseXML;
			var root_node = xmldoc.getElementsByTagName('latestversion').item(0);
			var latestVersion = root_node.firstChild.data;
			var thisVersion = "<?php echo GEOTRACK_VERSION; ?>";
			if (latestVersion != thisVersion) {
				if (confirm("A newer version (" + latestVersion + ") of this plugin is available!\nDo you want to download it?")) {
					var w = window.open("http://dadabase.de/weblog/archives/2006/05/23/wordpress-geotrack-plugin","download");
					w.focus();
				}
			}
			else {
				alert("Your version (" + thisVersion + ") is already up to date.");
			}
		} else {
			alert('An error has occured!');
		}
	}
	update_running = false;
}
		
		</script>
		<form method="post">
		<h2>WP/GeoTrack Plugin Options</h2>
		<fieldset name="set1" class="options">
			<legend>General Plugin Options</legend>
			<table class="editform optiontable">
				<tr valign="top">
					<th scope="row">List length</th>
					<td><input type="text" name="gt_list_length" value="<?php echo $options['gt_list_length']; ?>" size="3" /><br />
					Number of log-entries to display in our list and in your map.</td>
				</tr>
				<tr valign="top">
					<th scope="row">Top-List length</th>
					<td><input type="text" name="gt_toplist_length" value="<?php echo $options['gt_toplist_length']; ?>" size="3" /><br />
					Number of Top-Sites and Top-Referres to display in your list.</td>
				</tr>
				<tr valign="top">
					<th scope="row">Cleanup days</th>
					<td><input type="text" name="gt_cleanup_days" value="<?php echo $options['gt_cleanup_days']; ?>" size="3" /><br />
					Clean up database automatically after displaying the list.<br />
					0 will never do any cleanup, you might want to purge old entries manually from your db<br />
					Any number x will delete entries that are older than x days.</td>
				</tr>
				<tr valign="top">
					<th scope="row">Referrers only</th>
					<td><input type="checkbox" name="gt_referrers_only" value="1"<?php if ($options['gt_referrers_only']) echo ' checked="checked"'; ?> /><br />
					If ckecked, only request that have an external referrer will displayed in your list.</td>
				</tr>
				<tr valign="top">
					<th scope="row">Date format</th>
					<td><input type="text" name="gt_dateformat" value="<?php echo $options['gt_dateformat']; ?>" size="20" /><br />
					The dateformat for displaying the date in your list, see <a href="http://dev.mysql.com/doc/refman/4.1/en/date-and-time-functions.html">http://dev.mysql.com/doc/refman/4.1/en/date-and-time-functions.html</a>, eg:<br />
					<code>%d.%m.%Y %H:%i</code> =&gt; <code>24.02.2006 14:59</code> (little endian)<br />
					<code>%m/%d/%Y %h:%i %p</code> =&gt; <code>02/24/2006 02:59 PM</code> (american middle endian)</td>
				</tr>
				<tr valign="top">
					<th scope="row">Exclude user agents</th>
					<td><input type="text" name="gt_exclude_user_agents" value="<?php echo str_replace('|', ' ', $options['gt_exclude_user_agents']); ?>" size="90" /><br />
					List of space-seperated user agents to exclude from your tracking. E.g. "bot" will exclude every user agent that contains "bot" in its name, e.g. "Googlebot" and "MSN-Bot". Case insensitive.</td>
				</tr>
				<tr valign="top">
					<th scope="row">Debug</th>
					<td><input type="checkbox" name="gt_debug" value="1"<?php if ($options['gt_debug']) echo ' checked="checked"'; ?> /><br />
					Display debugging information (as comments in your HTML code)</td>
				</tr>
			</table>
		</fieldset>
		<fieldset name="set2" class="options">
			<legend>Google Map Options</legend>
			<table class="editform optiontable">
				<tr valign="top">
					<th scope="row">Google Map Key</th>
					<td><input type="text" name="gt_google_map_key" value="<?php echo $options['gt_google_map_key']; ?>" size="90" /><br />
					If you want to use Google-Maps, you have to enter your Maps API key here.
					If you don't have a key yet, you can get one for your site here: <br /><a href="http://www.google.de/apis/maps/signup.html">http://www.google.de/apis/maps/signup.html</a> (Google Account required)</td>
				</tr>
				<tr valign="top">
					<th scope="row">Google Map Center (Latitude / Longitude)</th>
					<td><input type="text" name="gt_google_map_lat" value="<?php echo $options['gt_google_map_lat']; ?>" size="10" /> /
					<input type="text" name="gt_google_map_long" value="<?php echo $options['gt_google_map_long']; ?>" size="10" /><br />
					Your current guessed location: <?php echo $_REQUEST['record']->city .' ('. $_REQUEST['record']->country_name . ') ('. $_REQUEST['record']->latitude . '/' . $_REQUEST['record']->longitude . ')'; ?></td>
				</tr>
				<tr valign="top">
					<th scope="row">Google Map Scale</th>
					<td><input type="text" name="gt_google_map_scale" value="<?php echo $options['gt_google_map_scale']; ?>" size="2" /><br />
					Set the initial scale of your Google map. Enter a number between 1 (complete world is visible) and 15 (street level).<br />
					4 might be a good starting point, and street level is not really useful because we only have city level information in our database.</td>
				</tr>
			</table>
		</fieldset>
		
		<p class="submit">
			<input type="submit" name="install_db" value="Install/Update MySQL Database Tables" />
			<input type="submit" name="update_db" id="update_db" value="Install/Update Geo-IP Database" onclick="updateGeoDB(); return false;" />
			<input type="submit" name="info_update" value="Update options" />
		</p>
		<h3>More actions:</h3>
		<p>
			<input type="submit" name="restore_defaults" value="Restore defaults" />
			<input type="submit" name="check_for_update" value="Check for plugin update" onclick="checkForUpdate(); return false;" />
		</p>
	</div><?php
}

// add the functions to WP
add_action('init', 'wp_geotrack_init');
add_action('wp_head', 'wp_geotrack_log');
add_action('rss_head', 'wp_geotrack_log');
add_action('rdf_ns', 'wp_geotrack_log');
add_action('atom_ns', 'wp_geotrack_log');
add_action('rss2_ns', 'wp_geotrack_log');
add_filter('the_content', 'wp_geotrack_content', 15);
add_action('admin_menu', 'wp_geotrack_add_menu');

?>